@@ -92,14 +92,14 @@ def test_internal_routing(subject, router):
                   _assert(
                       x[RuleConst.PROCESSED] and
                       len(x[RuleConst.PROCESSING]) == 1
-                  )
+                  ) and print(x)
     )
     results_rx_factory().subscribe(
         lambda x: x[RuleConst.RULE_NAME] == 'test-rule-filters-fails' and
                   _assert(
                       not x[RuleConst.PROCESSED] and
                       len(x[RuleConst.PROCESSING]) == 0
-                  )
+                  ) and print(x)
     )
 
     router.route("test-message", subject, {})
